/*
 * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

/**
 * @file doca_gpunetio_dev_buf.cuh
 * @page DOCA GPUNetIO CUDA Device functions
 * @defgroup DOCAGPUNETIO DOCA GPUNetIO engine
 * DOCA GPUNetio device library header to be included in CUDA .cu files.
 * All functions listed here must be called from a GPU CUDA kernel, they won't work from CPU.
 * All functions listed here should be considered as experimental.
 * For more details please refer to the user guide on DOCA devzone.
 *
 * @{
 */
#ifndef DOCA_GPUNETIO_DEVICE_BUF_H
#define DOCA_GPUNETIO_DEVICE_BUF_H

#include <stdint.h>
#include <doca_gpunetio.h>
#include <doca_error.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Opaque structures representing GPU handlers for DOCA buffers.
 */
struct doca_gpu_buf;
struct doca_gpu_buf_arr;

/*********************************************************************************************************************
 * DOCA buffer management
 *********************************************************************************************************************/

/**
 * @brief Retrieve a specific doca_gpu_buf pointer from doca_gpu_buf_arr object.
 *
 * @param [in] buf_arr_gpu
 * doca_gpu_buf_arr object.
 * @param [in] doca_gpu_buf_idx
 * Index of the doca_gpu_buf to retrieve from the receive queue.
 * @param [out] buf
 * doca_gpu_buf pointer.
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_INVALID_VALUE - arguments are invalid.
 *
 */
__device__ doca_error_t doca_gpu_dev_buf_get_buf(const struct doca_gpu_buf_arr *buf_arr_gpu, const uint64_t doca_gpu_buf_idx, struct doca_gpu_buf **buf);

/**
* @brief Retrieve memory address of the packet stored in the doca_gpu_buf.
*
* @param [in] buf
* doca_gpu_buf
* @param [out] addr
* Memory address of the packet in the doca_gpu_buf object
*
* @return
* DOCA_SUCCESS - in case of success.
* doca_error code - in case of failure:
* - DOCA_ERROR_INVALID_VALUE - arguments are invalid.
*
*/
__device__ doca_error_t doca_gpu_dev_buf_get_addr(const struct doca_gpu_buf *buf, uintptr_t *addr);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* DOCA_GPUNETIO_DEVICE_BUF_H */

/** @} */